//TEST PROGRAM for CRC routine
//Copyright (c) 1999 by Matthias Benkmann
//permission granted to use this code in commercial and uncommercial projects.

//mail questions and comments to mbenkmann@gmx.de

#include <string.h>
#include <conio.h>
#include <stdlib.h>
#include <iostream.h>
#include <stdio.h>

#include "crcpng.h"
#include "crc.h"
#include "stdmacs.h"

//SPEED ISSUES: Don't be fooled into believing that the bitwiseCRC class
//is slower than the make_crc_table() and crc() functions because of the
//"bitwise" in the name. Bitwise processing
//is an OPTION not a requirement of the bitwiseCRC class. The constructor of
//the bitwiseCRC class uses its own make_crc_table function to create a
//look-up table that is used by the ProcessByte() method. If you use
//ProcessByte() instead of ProcessBit() the bitwiseCRC class works exactly
//like crc().

int main()
begin
  char Test[2048];

  cout<<"Test 1: default 32bit CRC polynomial\n";
  cout<<"Enter Text to get CRC from: ";
  gets(Test);

  cout<<'\n';

  //The following part is just for comparison with the CRC code from the
  //.PNG specs. It is unrelated to bitwiseCRC. If you delete the following
  //3 instructions you can remove the crcpng.h and crcpng.cpp files.
  begin
      make_crc_table();
      unsigned long cc=crc(Test,strlen(Test));
      cout<<"CRC algo from PNG specs returns (bitwise complement): "<<(cc bXOR -1)<<"\n";
  end;



  int Length=32;
  int Round=2;
  bitwiseCRC CRC1;  //create bitwiseCRC object with default polynomial

  repeat
    char* Ptr=Test;
    while (*Ptr!=0) /*do*/ CRC1.ProcessByte(*Ptr++);
  
    cout<<"\nProcessByte algo returns: "<< (unsigned long)CRC1 <<"\n\n";
  
    cout<<"Now calculating CRC bitwise. The above CRC from ProcessByte will\n";
    cout<<"be appended to the data so the result must be 0 if ProcessByte() and\n";
    cout<<"ProcessBit() are consistent (they are supposed to)...\n";
  
    unsigned long cc=CRC1;
  
    CRC1.Reset();   //prepare for new CRC calculation
    Ptr=Test;
    while (*Ptr!=0) //do
    begin
      byte B=*Ptr;
      for (int count=8; --count>=0;) //do
      begin
        CRC1.ProcessBit(B);
        B=B shr 1;
      end;
      Ptr++;
    end;
  
    int i;
    for (i=Length; i>0; i--) //do
    begin
      CRC1.ProcessBit(cc);
      cc=cc shr 1;
    end;
  
    cout<<"ProcessBit algo returns (should be 0): "<<(unsigned long)CRC1<<'\n';
  
    cout<<'\n';
    cout<<"Test "<<Round++<<": user defined CRC. If you don't know what to use, try the \n";
    cout<<"16bit CCITT CRC used by floppy controllers (0x11021) or a 16bit CRC\n";
    cout<<"used in an IBM protocol (0x11005). The default polynomial used\n";
    cout<<"if you enter 0 is 0x104c11db7.\n";
    cout<<"ATTENTION!! You must enter the polynomial without MSB so you would\n";
    cout<<"enter 0x1005 for the IBM polynomial.\n\n";
  
    char Buf[256];
    cout<<"Enter CRC polynomial to use (without MSB!): ";
    gets(Buf);
    cc=strtol(Buf,NIL,0);

    Length=32;
    if cc==0 then cc=0x04c11db7; else
    begin
      cout<<"Enter length of polynomial (i.e. position of dropped MSB): ";
      gets(Buf);
      Length=strtol(Buf,NIL,0);
    end;

    //reverse the value so that MSB becomes LSB
    unsigned long cc2=0;
    for (i=Length; i>0; i--) //do
    begin
      cc2=cc2 shl 1;
      if (cc bAND 1)==1 then cc2++;
      cc=cc shr 1;
    end;

    CRC1.Init(cc2);  //set new CRC value.
  until (false);
end;